;;;                                                                    ;
;;;  atableF2G.LSP                                                     ;
;;;                                                                    ;
;;;  Copyright 2005 by Alexander Shchetinin. All Rights Reserved.      ;
;;;                                                                    ;
;;;  You are hereby granted permission to use, copy and modify this    ;
;;;  software without charge, provided you do so exclusively for       ;
;;;  your own use or for use by others in your organization in the     ;
;;;  performance of their normal duties, and provided further that     ;
;;;  the above copyright notice appears in all copies and both that    ;
;;;  copyright notice and the limited warranty and restricted rights   ;
;;;  notice below appear in all supporting documentation.              ;
;;;                                                                    ;
;;;  Incorporation of any part of this software into other software,   ;
;;;  except when such incorporation is exclusively for your own use    ;
;;;  or for use by others in your organization in the performance of   ;
;;;  their normal duties, is prohibited without the prior written      ;
;;;  consent of author.                                                ;
;;;                                                                    ;
;;;  Copying, modification and distribution of this software or any    ;
;;;  part thereof in any form except as expressly provided herein is   ;
;;;  prohibited without the prior written consent of author.           ;
;;;                                                                    ;
;;;  AUTHOR PROVIDES THIS SOFTWARE "AS IS" AND WITH ALL FAULTS.        ;
;;;  AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF             ;
;;;  MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTHOR,         ;
;;;  DOES NOT WARRANT THAT THE OPERATION OF THE SOFTWARE               ;
;;;  WILL BE UNINTERRUPTED OR ERROR FREE.                              ;
;;;                                                                    ;
;;;--------------------------------------------------------------------;
;;;       Function:  atableF2G                                         ;
;;;                                                                    ;
;;;    Description:  This draw forms by GOST 21.101-97.                ;
;;;                                                                    ;
;;;                                                                    ;
;;;                                                                    ;
;;;                  Required Application: ATable for AutoCAD          ;
;;;                                                                    ;
;;;      Arguments:  None                                              ;
;;;                                                                    ;
;;; Returned Value:  None	                                       ;
;;;                                                                    ;
;;;          Usage:  Load atableF2G.lsp in AutoCAD by command Appload. ;
;;;                  Run ATable for AutoCAD. Display ATable editor     ;
;;;                  Enter option Add-ons in command line. Select F2G  ;
;;;                  option and follow instructions in command line.   ;
;;;                                                                    ;
;;;--------------------------------------------------------------------;

;(setq atableF2Glocal
;  (list 
;    (cons 20 "Alxd ATable   !")
;    (cons 30 "  Alxd ATable   !")
;    (cons 60 "3Form")
;    (cons 61 "4Form")
;    (cons 69 "eXit")
;    (cons 70 "\n  [3Form/4Form/eXit]: ")
;  )
;)

;You can localize this application. Only change text strings below.
(setq atableF2Glocal
  (list 
    (cons 20 "Alxd ATable is closed or unavailable!")
    (cons 30 "Active sheet of Alxd ATable is closed or unavailable!")
    (cons 60 "3Form")
    (cons 61 "4Form")
    (cons 69 "eXit")
    (cons 70 "\nEnter an option [3Form/4Form/eXit]: ")
  )
)

(defun getLocal ( slocal nlocal / )
  (cdr (assoc nlocal slocal))
)

(defun getActiveSheet(atObject / atEditor atSS atActiveSheet wi)
   (setq
      atEditor (vlax-get-property atObject 'AlxdEditor)
      atSS (vlax-get-property atEditor 'AlxdSpreadSheets)
      wi (vlax-get-property atSS 'Active)
      atActiveSheet (vlax-get-property atSS 'Items wi)
   )
  (vlax-release-object atSS)
  (vlax-release-object atEditor)
  atActiveSheet
)

(defun alxd:multiSelect( slist atActiveSheet / atSsets atSset wj)
  (setq
    atSsets (vlax-get-property atActiveSheet 'Selections)
    wj 0
  )
  (vlax-put-property atSsets 'Count (length slist))
  (repeat (length slist)
    (setq 
      atSset (vlax-get-property atSsets 'Items wj)
      temp (nth wj slist)
    )
    (vlax-invoke-method atSset 'Make (nth 0 temp) (nth 1 temp) (nth 2 temp) (nth 3 temp) (nth 4 temp))
    (vlax-release-object atSset)
    (setq wj (1+ wj))
  )
  (vlax-release-object atSsets)
)

(defun alxd:setHorBorder( acol arow atBorders / atBorder)
  (setq atBorder (vlax-get-property atBorders 'Horizontal acol arow))
  (vlax-put-property atBorder 'Color 0)
  (vlax-put-property atBorder 'Weight -2)
  (vlax-release-object atBorder)
)

(defun alxd:setText( acol arow text atCells / atCell)
  (setq atCell (vlax-get-property atCells 'Items acol arow))
  (vlax-put-property atCell 'Contain text)
  (vlax-put-property atCell 'TextHorizontalAlignment 2)
  (vlax-put-property atCell 'TextFit 4)
  (vlax-release-object atCell)
)

(defun alxd:resetJoin( atActiveSheet / atSsets atSset atRows atCols atCells wi wj)
  (setq 
    atSsets (vlax-get-property atActiveSheet 'Selections)
    atCols (vlax-get-property atActiveSheet 'Columns)
    atRows (vlax-get-property atActiveSheet 'Rows)
    atCells (vlax-get-property atActiveSheet 'Cells)
    wi (vlax-get-property atCols 'Count)
    wj (vlax-get-property atRows 'Count)
  )
  (vlax-put-property atSsets 'Count 1)
  (setq atSset (vlax-get-property atSsets 'Items 0))
  (vlax-invoke-method atSset 'Make 0 0 (1- wi) (1- wj) 0)
  (vlax-invoke-method atCells 'Unjoin)

  (vlax-release-object atCells)
  (vlax-release-object atRows)
  (vlax-release-object atCols)
  (vlax-release-object atSset)
  (vlax-release-object atSsets)
)

(defun alxd:drawF4( atActiveSheet / atStyle atCols atCol atRows atRow atSsets atCells atBorders wi wj temp)
  (setq 
    atStyle (vlax-get-property atActiveSheet 'Style)
    atSsets (vlax-get-property atActiveSheet 'Selections)
    atRows (vlax-get-property atActiveSheet 'Rows)
    atCols (vlax-get-property atActiveSheet 'Columns)
    atCells (vlax-get-property atActiveSheet 'Cells)
    atBorders (vlax-get-property atActiveSheet 'Borders)
  )  

  ;setup Style
  (vlax-put-property atStyle 'DefaultSize 5)
  (vlax-put-property atStyle 'Justify 3)

  (vlax-put-property atStyle 'TextColor 7)
  (vlax-put-property atStyle 'TextWeight 18)
  (vlax-put-property atStyle 'TextWidthFactor 0.7)
  (vlax-put-property atStyle 'TextMarginLeft 1)
  (vlax-put-property atStyle 'TextMarginBottom 1)
  (vlax-put-property atStyle 'TextMarginRight 1)
  (vlax-put-property atStyle 'TextMarginTop 1)

  (vlax-put-property atStyle 'HorBorderColor 7)
  (vlax-put-property atStyle 'HorBorderWeight 18)
  (vlax-put-property atStyle 'VerBorderColor 0)
  (vlax-put-property atStyle 'VerBorderWeight -2)

  (vlax-put-property atActiveSheet 'Style atStyle)

  ;setup Grid
  (vlax-put-property atCols 'Count 11)
  (vlax-put-property atRows 'Count 11)

  ;setup Columns
  (setq 
    wi (list 10 10 10 10 15 10 70 15 5 10 20)
    wj 0
  )  
  (repeat (length wi)
    (setq atCol (vlax-get-property atCols 'Items wj))
    (vlax-put-property atCol 'Size (nth wj wi))
    (vlax-release-object atCol)
    (setq wj (1+ wj))
  )

  ;setup Rows
  ;by Style

  ;setup Joins
  (setq wi (list '(6 0 10 2 0) '(6 3 6 7 0) '(6 8 6 10 0) '(8 3 9 3 0) '(7 4 7 6 0) '(8 4 9 6 0) '(10 4 10 6 0) '(7 7 8 7 0) '(9 7 10 7 0) '(7 8 10 10 0)))
  (alxd:multiSelect wi atActiveSheet)
  (vlax-invoke-method atCells 'Join :vlax-false)

  (setq wi (list '(0 5 1 10 0) '(2 5 3 10 0)))
  (alxd:multiSelect wi atActiveSheet)
  (vlax-invoke-method atCells 'JoinByRow :vlax-false)

  (vlax-invoke-method atSsets 'SelectCell 0 0)

  ;setup Cells
  (alxd:setText 0 4 "." atCells)
  (alxd:setText 1 4 ". ." atCells)
  (alxd:setText 2 4 "" atCells)
  (alxd:setText 3 4 "N%%d ." atCells)
  (alxd:setText 4 4 "." atCells)
  (alxd:setText 5 4 "" atCells)
  (alxd:setText 7 3 "" atCells)
  (alxd:setText 8 3 "" atCells)
  (alxd:setText 10 3 "" atCells)
  (alxd:setText 7 7 "" atCells)
  (alxd:setText 9 7 "" atCells)

  ;setup Borders
  (setq 
    wi (list '(0 4) '(0 5) '(1 4) '(1 5) '(2 4) '(2 5) '(2 4) '(2 5) '(3 4) '(3 5) '(4 4) '(4 5) '(5 4) '(5 5) '(6 8) '(7 8) '(8 8) '(9 8) '(10 8) '(7 7) '(8 7) '(9 7) '(10 7) '(7 4) '(8 4) '(9 4) '(10 4) '(6 3) '(7 3) '(8 3) '(9 3) '(10 3) '(0 0) '(1 0) '(2 0) '(3 0) '(4 0) '(5 0) '(6 0) '(7 0) '(8 0) '(9 0) '(10 0))
    wj 0
  )
  (repeat (length wi)
    (setq temp (nth wj wi))
    (alxd:setHorBorder (nth 0 temp) (nth 1 temp) atBorders)
    (setq wj (1+ wj))
  )

  (vlax-invoke-method atActiveSheet 'Recalculate)
  (vlax-invoke-method atActiveSheet 'RedrawBlockDefinition 0 0)

  (vlax-invoke-method  atSsets 'Redraw)

  (vlax-release-object atSsets)
  (vlax-release-object atStyle)
  (vlax-release-object atBorders)
  (vlax-release-object atCells)
  (vlax-release-object atCols)
  (vlax-release-object atRows)
)

(defun alxd:drawF3( atActiveSheet / atStyle atCols atCol atRows atRow atSsets atCells atBorders wi wj temp)
  (setq 
    atStyle (vlax-get-property atActiveSheet 'Style)
    atSsets (vlax-get-property atActiveSheet 'Selections)
    atRows (vlax-get-property atActiveSheet 'Rows)
    atCols (vlax-get-property atActiveSheet 'Columns)
    atCells (vlax-get-property atActiveSheet 'Cells)
    atBorders (vlax-get-property atActiveSheet 'Borders)
  )  
  ;setup Style
  (vlax-put-property atStyle 'DefaultSize 5)
  (vlax-put-property atStyle 'Justify 3)

  (vlax-put-property atStyle 'TextColor 7)
  (vlax-put-property atStyle 'TextWeight 18)
  (vlax-put-property atStyle 'TextWidthFactor 0.7)
  (vlax-put-property atStyle 'TextMarginLeft 1)
  (vlax-put-property atStyle 'TextMarginBottom 1)
  (vlax-put-property atStyle 'TextMarginRight 1)
  (vlax-put-property atStyle 'TextMarginTop 1)

  (vlax-put-property atStyle 'HorBorderColor 7)
  (vlax-put-property atStyle 'HorBorderWeight 18)
  (vlax-put-property atStyle 'VerBorderColor 0)
  (vlax-put-property atStyle 'VerBorderWeight -2)

  (vlax-put-property atActiveSheet 'Style atStyle)

  ;setup Grid
  (vlax-put-property atCols 'Count 10)
  (vlax-put-property atRows 'Count 11)

  ;setup Columns
  (setq 
    wi (list 10 10 10 10 15 10 70 15 15 20)
    wj 0
  )  
  (repeat (length wi)
    (setq atCol (vlax-get-property atCols 'Items wj))
    (vlax-put-property atCol 'Size (nth wj wi))
    (vlax-release-object atCol)
    (setq wj (1+ wj))
  )

  ;setup Rows
  ;by Style

  ;setup Joins
  (setq wi (list '(6 0 9 1 0) '(6 2 9 4 0) '(6 5 6 7 0) '(6 8 6 10 0) '(7 6 7 7 0) '(8 6 8 7 0) '(9 6 9 7 0) '(7 8 9 10 0)))
  (alxd:multiSelect wi atActiveSheet)
  (vlax-invoke-method atCells 'Join :vlax-false)

  (setq wi (list '(0 5 1 10 0) '(2 5 3 10 0)))
  (alxd:multiSelect wi atActiveSheet)
  (vlax-invoke-method atCells 'JoinByRow :vlax-false)

  (vlax-invoke-method atSsets 'SelectCell 0 0)

  ;setup Cells
  (alxd:setText 0 4 "." atCells)
  (alxd:setText 1 4 ". ." atCells)
  (alxd:setText 2 4 "" atCells)
  (alxd:setText 3 4 "N%%d ." atCells)
  (alxd:setText 4 4 "." atCells)
  (alxd:setText 5 4 "" atCells)
  (alxd:setText 7 5 "" atCells)
  (alxd:setText 8 5 "" atCells)
  (alxd:setText 9 5 "" atCells)

  ;setup Borders
  (setq 
    wi (list '(0 4) '(0 5) '(1 4) '(1 5) '(2 4) '(2 5) '(2 4) '(2 5) '(3 4) '(3 5) '(4 4) '(4 5) '(5 4) '(5 5) '(6 8) '(7 8) '(8 8) '(9 8) '(7 6) '(8 6) '(9 6) '(6 5) '(7 5) '(8 5) '(9 5) '(6 2) '(7 2) '(8 2) '(9 2) '(0 0) '(1 0) '(2 0) '(3 0) '(4 0) '(5 0) '(6 0) '(7 0) '(8 0) '(9 0))
    wj 0
  )
  (repeat (length wi)
    (setq temp (nth wj wi))
    (alxd:setHorBorder (nth 0 temp) (nth 1 temp) atBorders)
    (setq wj (1+ wj))
  )

  (vlax-invoke-method atActiveSheet 'Recalculate)
  (vlax-invoke-method atActiveSheet 'RedrawBlockDefinition 0 0)

  (vlax-invoke-method  atSsets 'Redraw)

  (vlax-release-object atSsets)
  (vlax-release-object atStyle)
  (vlax-release-object atBorders)
  (vlax-release-object atCells)
  (vlax-release-object atCols)
  (vlax-release-object atRows)
)

(defun interfaceF2G ( atObject / temp xx atActiveSheet)
  (setq temp T)

  (while temp

    (setq 
      cmdstr (getlocal atableF2Glocal 70)
      inistr (strcat (getlocal atableF2Glocal 60) " " (getlocal atableF2Glocal 61) " " (getlocal atableF2Glocal 69))
    )

    (initget 1 inistr)
    (setq xx (getkword cmdstr))
    (cond
      ((= xx (getlocal atableF2Glocal 60));3Form
        (setq atActiveSheet (getActiveSheet atObject))
        (alxd:resetJoin atActiveSheet)
        (alxd:drawF3 atActiveSheet)
      )
      ((= xx (getlocal atableF2Glocal 61));4Form
        (setq atActiveSheet (getActiveSheet atObject))
        (alxd:resetJoin atActiveSheet)
        (alxd:drawF4 atActiveSheet)
      )
      ((or (= xx nil) (= xx (getlocal atableF2Glocal 69))) (setq temp nil))
    );cond
  )
)

(defun c:atableF2G( / sstext atObject)
  (vl-load-com)

  ;get alxd atable object here
  (setq atObject (vlax-get-object "AlxdGrid.AlxdApplication"))

  ;check value
  (if (= atObject nil)
    (progn
      (alert (getlocal atableX2Glocal 20))
      (exit)
    )
  )

;debug string
;  (vlax-dump-object atObject)

  (vl-catch-all-apply 'interfaceF2G (list atObject))
  (vlax-release-object atObject) 
 
  (princ)
)
(princ "\natableF2G v1.2 (addon for ATable) - draw forms by GOST 21.101-97")
(princ)